//==============================================================================
// IOPINS - Handle some of the IO functions of the BAP
//==============================================================================
#include "globals.h"
#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

#define _LOW_ 						0
#define _HIGH_						1

#define _BA_PIN_MODE_IO_			0
#define _BA_PIN_MODE_FUNCTION_		1

#define _BA_PIN_MODE_INPUT_			0
#define _BA_PIN_MODE_OUTPUT_		1

#define _BA_ADC_MODE_SINGLE_		0
#define _BA_ADC_MODE_SCAN_			1

#define _BA_ADC_CLOCK_134_			0
#define _BA_ADC_CLOCK_70_			1

enum _PCR_SHADOW_REGS_ {_PCR1_SHADOW_, _PCR2_SHADOW_, _PCR3_SHADOW_, _PCR4_SHADOW_, cPCR5_SHADOW, _PCR6_SHADOW_, _PCR7_SHADOW_, _PCR8_SHADOW_, _PCR_SHADOW_COUNT_};
u8 bap_bpcrshadowregs [_PCR_SHADOW_COUNT_];

const PORTITEM	g_PortTable[] =
{
#ifdef _BAP28_
	{&IO.PDR5.BYTE, 0x1, (PUR_5 | PMR_5)},	// P0
	{&IO.PDR5.BYTE, 0x2, (PUR_5 | PMR_5)},	// P1
	{&IO.PDR5.BYTE, 0x4, (PUR_5 | PMR_5)},	// P2
	{&IO.PDR5.BYTE, 0x8, (PUR_5 | PMR_5)},	// P3
	{&IO.PDR5.BYTE, 0x10, (PUR_5 | PMR_5)},	// P4
	{&IO.PDR5.BYTE, 0x20, (PUR_5 | PMR_5)},	// P5
	{&IO.PDR5.BYTE, 0x40, 0},	// P6
	{&IO.PDR5.BYTE, 0x80, 0},	// P7
	{&IO.PDR8.BYTE, 0x1, 0},	// P8 - Does support pull-up but on P15?  so for now say no
	{&IO.PDR8.BYTE, 0x2, 0},	// P9
	{&IO.PDR8.BYTE, 0x4, 0},	// P10
	{&IO.PDR8.BYTE, 0x8, 0},	// P11
	{&IO.PDR8.BYTE, 0x10, 0},	// P12 - Does support pull-up on P10; so far now say no
	{&IO.PDR8.BYTE, 0x20, 0},	// P13
#if 1
	{&IO.PDR8.BYTE, 0x40, 0},	// P14
	{&IO.PDR8.BYTE, 0x80, 0},	// P15
#else
	{&IO.PDR2.BYTE, 0x02, 0},	// P14
	{&IO.PDR2.BYTE, 0x04, 0},	// P15
#endif	
	{&IO.PDR1.BYTE, 0x1, (PUR_1 | PMR_1)},	// P16
	{&IO.PDR1.BYTE, 0x2, (PUR_1 | PMR_1)},	// P17
	{&IO.PDR1.BYTE, 0x40, (PUR_1 | PMR_1)},	// P18
	{&IO.PDR1.BYTE, 0x80, (PUR_1 | PMR_1)},	// P19
	{0, 0, 0},	//20
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},
	{0, 0, 0},//30
	{0, 0, 0},
	{&IO.PDR1.BYTE, 0x10, 0},	// S_IN
	{&IO.PDR1.BYTE, 0x10, 0}	// S_OUT
#endif /* _BAP28_ */

// OK lets define Bap40 pins as well
#ifdef _BAP40_
	{&IO.PDR5.BYTE, 0x1, (PUR_5 | PMR_5)},	// P0
	{&IO.PDR5.BYTE, 0x2, (PUR_5 | PMR_5)},	// P1
	{&IO.PDR5.BYTE, 0x4, (PUR_5 | PMR_5)},	// P2
	{&IO.PDR5.BYTE, 0x8, (PUR_5 | PMR_5)},	// P3
	{&IO.PDR5.BYTE, 0x10, (PUR_5 | PMR_5)},	// P4
	{&IO.PDR5.BYTE, 0x20, (PUR_5 | PMR_5)},	// P5
	{&IO.PDR5.BYTE, 0x40, 0},	// P6
	{&IO.PDR5.BYTE, 0x80, 0},	// P7
	{&IO.PDR6.BYTE, 0x1, 0},	// P8
	{&IO.PDR6.BYTE, 0x2, 0},	// P9
	{&IO.PDR6.BYTE, 0x4, 0},	// P10
	{&IO.PDR6.BYTE, 0x8, 0},	// P11
	{&IO.PDR6.BYTE, 0x10, 0},	// P12
	{&IO.PDR6.BYTE, 0x20, 0},	// P13
	{&IO.PDR6.BYTE, 0x40, 0},	// P14
	{&IO.PDR6.BYTE, 0x80, 0},	// P15
	{&IO.PDR3.BYTE, 0x1,  0},	// P16
	{&IO.PDR3.BYTE, 0x2, 0},	// P17
	{&IO.PDR3.BYTE, 0x4, 0},	// P18
	{&IO.PDR3.BYTE, 0x8,  0},	// P19
	{&IO.PDR3.BYTE, 0x10, 0},	// P20
	{&IO.PDR3.BYTE, 0x20, 0},	// P21
	{&IO.PDR3.BYTE, 0x40, 0},	// P22
	{&IO.PDR3.BYTE, 0x80, 0},	// P23
	{&IO.PDR1.BYTE, 0x1, (PUR_1 | PMR_1)},	// P24
	{&IO.PDR1.BYTE, 0x2, (PUR_1 )},		// P25 - PMR does not match
	{&IO.PDR1.BYTE, 0x4, (PUR_1 | PMR_1)},	// P26
	{&IO.PDR2.BYTE, 0x8, (PUR_20)},	// P27
	{&IO.PDR1.BYTE, 0x10, (PUR_1 | PMR_1)},	// P28
	{&IO.PDR1.BYTE, 0x20, (PUR_1 | PMR_1)},	// P29
	{&IO.PDR1.BYTE, 0x40, (PUR_1 | PMR_1)},	// P30
	{&IO.PDR1.BYTE, 0x80, (PUR_1 | PMR_1)},	// P31
	{&IO.PDR2.BYTE, 0x2, 0},	// S_IN (P21)
	{&IO.PDR2.BYTE, 0x4, 0}	// S_OUT TXD(P22)

#endif
	
};	

//Pro 28 H8/3664F Pro 28 H8/3664F
//P0 P50/WKP0 PB0/AN0 P8 P80/FTCI P15/IRQ1
//P1 P51/WKP1 PB1/AN1 P9 P81/FTIOA P74/TMRIV
//P2 P52/WKP2 PB2/AN2 P10 P82/FTIOB
//P3 P53/WKP3 PB3/AN3 P11 P83/FTIOC
//P4 P54/WKP4 P12 P84/FTIOD P10/TMOW
//P5 P55/WKP5/ADTRG P13 P85 P20/SCK3
//P6 P56/SDA P14 P86 P21/RXD
//P7 P57/SCL P15 P87 P22/TXD
//P16 P11 PB7/AN7 P18 P16/IRQ2 PB5/AN5
//P17 P12 PB6/AN6 P19 P17/IRQ3 PB4/AN4



// initialize the Basic Atom Pro CPU
void bap_initialize (void)
{
	// disable IRQs
	IO.PMR1.BIT.IRQ0 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ1 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ2 = _BA_PIN_MODE_IO_;
	IO.PMR1.BIT.IRQ3 = _BA_PIN_MODE_IO_;
	// disable interrupts
	IENR1.BIT.IEN0 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN1 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN2 = _BA_PIN_MODE_IO_;
	IENR1.BIT.IEN3 = _BA_PIN_MODE_IO_;
	// set PORT5 wakeup pins as IO
	IO.PMR5.BIT.WKP0 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP1 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP2 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP3 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP4 = _BA_PIN_MODE_IO_;
	IO.PMR5.BIT.WKP5 = _BA_PIN_MODE_IO_;
	// set TMOW as IO pin
	IO.PMR1.BIT.TMOW = _BA_PIN_MODE_IO_;
	// set TXD as IO pin
	IO.PMR1.BIT.TXD = _BA_PIN_MODE_IO_;
	// set RXD as IO pin
	SCI3.SCR3.BIT.RE = _BA_PIN_MODE_IO_;
	SCI3.SSR.BYTE = 0;			// clear out any status

	// set SCK3 as IO pin
	SCI3.SCR3.BIT.CKE = _BA_PIN_MODE_IO_;
	SCI3.SMR.BIT.COM = _BA_PIN_MODE_IO_;

	SCI3.SCR3.BIT.RIE = _FALSE_;		// disable interrupts when we receive a character or error
	SCI3.SCR3.BIT.TIE = _FALSE_;		// disable interrupts when ready for next byte to transmit
	SCI3.SCR3.BIT.TEIE = _FALSE_;		// For now don't need to enable interrupt when transmit complete...


	IIC2.ICIER.BYTE = 0;				// Make sure I2C is not going to generate any interrupts.
	IIC2.ICCR1.BIT.ICE = 0;				// Likewise it is disabled...

	// set PORT pins as input
	bap_bpcrshadowregs [_PCR1_SHADOW_] = 0;
	bap_bpcrshadowregs [_PCR2_SHADOW_] = 0;
	bap_bpcrshadowregs [cPCR5_SHADOW] = 0;
	bap_bpcrshadowregs [_PCR7_SHADOW_] = 0;
	bap_bpcrshadowregs [_PCR8_SHADOW_] = 0;
	// set PORT pins from shadow registers
	IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
	IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
	IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
	IO.PCR7 = bap_bpcrshadowregs [_PCR7_SHADOW_];
	IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
	// disable input pullups
	IO.PUCR1.BYTE = 0x00;
	IO.PUCR5.BYTE = 0x00;

	#ifdef _BAP28_
	// set RXD as IO pin
	SCI3.SCR3.BIT.RE = _BA_PIN_MODE_IO_;
	// disable FTIO
	TW.TIOR0.BIT.IOA = 0x00;
	TW.TIOR0.BIT.IOB = 0x00;
	TW.TIOR1.BIT.IOC = 0x00;
	TW.TIOR1.BIT.IOD = 0x00;
	// disable analog to digital
	AD.ADCSR.BIT.ADST = 0;
	// disable timer interrupts
	IENR1.BIT.IENTA = 0; // timera
	// disable timer counting
	TW.TMRW.BIT.CTS = 0; // timerw
	#endif
}



// pause 0.5 microseconds
void pauseus (u32 _lpause)
{
	u32 lpause;
	for (lpause=0; lpause<_lpause; lpause++);
}

// pause milliseconds
void pause (u32 _lpause)
{
	pauseus ((_lpause*2000));
}





// make pin an input
void input (u8 _bpin, bool fPullup)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM)))
	{
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
		{
			// See if there is a PMR register associated with this pin.
			if ((pi.bPU_PM) & 0xf)
			{
				volatile register unsigned char *pPMR = &IO.PMR1.BYTE + (pi.bPU_PM & 0xf) - 1;	// get the right PMR register
				*pPMR &= ~(pi.bMask);
			}
			
			// Next lets update PDR/PCR...
			u8 bIOPortIndex = (pi.pPDR - &IO.PDR1.BYTE);
			volatile unsigned char *pPCR = &IO.PCR1 + bIOPortIndex;
			bap_bpcrshadowregs[bIOPortIndex] &= ~(pi.bMask);				// Update shadow mask to say that the pin is now an input pin
			*pPCR = bap_bpcrshadowregs[bIOPortIndex];
			
			// Now lets see if we have a PUCR register (pull-up) to update for this pin
			if ((pi.bPU_PM) >> 4)
			{
				volatile register unsigned char *pPUCR = &IO.PUCR1.BYTE + (pi.bPU_PM >> 4) - 1;	// get the right PUCR register
				if (fPullup)
					*pPUCR |= pi.bMask;
				else
					*pPUCR &= ~(pi.bMask);
			}
			
		}
	}	
			
#if 0		
	switch (_bpin)
	{
		#ifdef _BAP28_
		case 0:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP0 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xFE;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 1:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP1 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xFD;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 2:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP2 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xFB;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 3:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP3 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xF7;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 4:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP4 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xEF;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 5:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP5 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xDF;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 6:
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xBF;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 7:
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0x7F;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 8:
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xFE;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 9:
			// disable FTIOA
			TW.TIOR0.BIT.IOA = 0x00;
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xFD;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 10:
			// disable FTIOB
			TW.TIOR0.BIT.IOB = 0x00;
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xFB;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 11:
			// disable FTIOC
			TW.TIOR1.BIT.IOC = 0x00;
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xF7;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 12:
			// disable FTIOD
			TW.TIOR1.BIT.IOD = 0x00;					
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xEF;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 13:
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xDF;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 14:
			bap_bpcrshadowregs [_PCR2_SHADOW_] &= 0xFD;
			IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
			break;
		case 15:
			bap_bpcrshadowregs [_PCR2_SHADOW_] &= 0xFB;
			IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
			break;
		case 16:
			// disable input pullup
			IO.PUCR1.BIT.B4 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xFD;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 17:
			// disable input pullup
			IO.PUCR1.BIT.B5 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xFB;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 18:
			// disable IRQ
			IO.PMR1.BIT.IRQ2 = _BA_PIN_MODE_IO_;
			// disable input pullup
			IO.PUCR1.BIT.B6 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xBF;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 19:
			// disable IRQ
			IO.PMR1.BIT.IRQ3 = _BA_PIN_MODE_IO_;
			// disable input pullup
			IO.PUCR1.BIT.B7 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0x7F;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case _S_IN_:
			// disable IRQ
			IO.PMR1.BIT.IRQ0 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xEF;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		#endif
		#ifdef _BAP40_
		case 0:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP0 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xFE;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 1:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP1 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xFD;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 2:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP2 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xFB;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 3:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP3 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xF7;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 28:
			// disable IRQ
			IO.PMR1.BIT.IRQ0 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xEF;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 4:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP4 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xEF;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 5:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP5 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] &= 0xDF;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 6:
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xBF;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 7:
			bap_bpcrshadowregs [_PCR8_SHADOW_] &= 0xDF;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 8:
			// disable FTIOA0
			TZ.TOER.BIT.EB1 = 1;
			TZ0.TIORA.BIT.IOA = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0xFE;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 9:
			// disable FTIOB0
			TZ.TOER.BIT.EB1 = 1;
			TZ0.TIORA.BIT.IOB = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0xFD;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 10:
			// disable FTIOC0
			TZ.TOER.BIT.EB1 = 1;
			TZ0.TIORC.BIT.IOC = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0xFB;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 11:
			// disable FTIOD0
			TZ.TOER.BIT.EB1 = 1;
			TZ0.TIORC.BIT.IOD = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0xF7;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 12:
			// disable FTIOA1
			TZ.TOER.BIT.EB1 = 1;
			TZ1.TIORA.BIT.IOA = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0xEF;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 13:
			// disable FTIOB1
			TZ.TOER.BIT.EB1 = 1;
			TZ1.TIORA.BIT.IOB = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0xDF;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 14:
			// disable FTIOC1
			TZ.TOER.BIT.EB1 = 1;
			TZ1.TIORC.BIT.IOC = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0xBF;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 15:
			// disable FTIOD1
			TZ.TOER.BIT.EB1 = 1;
			TZ1.TIORC.BIT.IOD = 0x00;
			bap_bpcrshadowregs [_PCR6_SHADOW_] &= 0x7F;
			IO.PCR6 = bap_bpcrshadowregs [_PCR6_SHADOW_];
			break;
		case 16:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0xFE;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 17:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0xFD;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 18:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0xFB;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 19:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0xF7;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 20:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0xEF;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 21:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0xDF;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 22:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0xBF;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 23:
			bap_bpcrshadowregs [_PCR3_SHADOW_] &= 0x7F;
			IO.PCR3 = bap_bpcrshadowregs [_PCR3_SHADOW_];
			break;
		case 24:
			// set TMOW as IO pin
			IO.PMR1.BIT.TMOW = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xFE;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 25:
			// set PWM as IO pin
			IO.PMR1.BIT._PWM = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xFD;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 26:
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xFB;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 27:
			bap_bpcrshadowregs [_PCR2_SHADOW_] &= 0xF7;
			IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
		case 29:
			// disable IRQ
			IO.PMR1.BIT.IRQ1 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xDF;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 30:
			// disable IRQ
			IO.PMR1.BIT.IRQ2 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0xBF;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 31:
			// disable IRQ
			IO.PMR1.BIT.IRQ3 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] &= 0x7F;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case _S_IN_:
			bap_bpcrshadowregs [_PCR2_SHADOW_] &= 0xFB;
			IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
			break;
		#endif
	}
#endif	
}

// make pin an output
void output (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM)))
	{
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
		{
			// See if there is a PMR register associated with this pin.
			if ((pi.bPU_PM) & 0xf)
			{
				volatile register unsigned char *pPMR = &IO.PMR1.BYTE + (pi.bPU_PM & 0xf) - 1;	// get the right PMR register
				*pPMR &= ~(pi.bMask);
			}
			
			// Next lets update PDR/PCR...
			u8 bIOPortIndex = (pi.pPDR - &IO.PDR1.BYTE);
			volatile unsigned char *pPCR = &IO.PCR1 + bIOPortIndex;
			bap_bpcrshadowregs[bIOPortIndex] |= pi.bMask;				// Ok update shadaw mask to say it is output
			*pPCR = bap_bpcrshadowregs[bIOPortIndex];
		}
	}	
			

#if 0
	switch (_bpin)
	{
		#ifdef _BAP28_
		case 0:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP0 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x01;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 1:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP1 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x02;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 2:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP2 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x04;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 3:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP3 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x08;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 4:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP4 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x10;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 5:
			// set PORT5 wakeup pins as IO
			IO.PMR5.BIT.WKP5 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x20;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 6:
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x40;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 7:
			bap_bpcrshadowregs [cPCR5_SHADOW] |= 0x80;
			IO.PCR5 = bap_bpcrshadowregs [cPCR5_SHADOW];
			break;
		case 8:
			bap_bpcrshadowregs [_PCR8_SHADOW_] |= 0x01;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 9:
			// disable FTIOA
			TW.TIOR0.BIT.IOA = 0x00;
			bap_bpcrshadowregs [_PCR8_SHADOW_] |= 0x02;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 10:
			// disable FTIOB
			TW.TIOR0.BIT.IOB = 0x00;
			bap_bpcrshadowregs [_PCR8_SHADOW_] |= 0x04;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 11:
			// disable FTIOC
			TW.TIOR1.BIT.IOC = 0x00;
			bap_bpcrshadowregs [_PCR8_SHADOW_] |= 0x08;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 12:
			// disable FTIOD
			TW.TIOR1.BIT.IOD = 0x00;
			bap_bpcrshadowregs [_PCR8_SHADOW_] |= 0x10;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 13:
			bap_bpcrshadowregs [_PCR8_SHADOW_] |= 0x20;
			IO.PCR8 = bap_bpcrshadowregs [_PCR8_SHADOW_];
			break;
		case 14:
			bap_bpcrshadowregs [_PCR2_SHADOW_] |= 0x02;
			IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
			break;
		case 15:
			bap_bpcrshadowregs [_PCR2_SHADOW_] |= 0x04;
			IO.PCR2 = bap_bpcrshadowregs [_PCR2_SHADOW_];
			break;
		case 16:
			// disable input pullup
			IO.PUCR1.BIT.B4 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] |= 0x02;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 17:
			// disable input pullup
			IO.PUCR1.BIT.B5 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] |= 0x04;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 18:
			// disable IRQ
			IO.PMR1.BIT.IRQ2 = _BA_PIN_MODE_IO_;
			// disable input pullup
			IO.PUCR1.BIT.B6 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] |= 0x40;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case 19:
			// disable IRQ
			IO.PMR1.BIT.IRQ3 = _BA_PIN_MODE_IO_;
			// disable input pullup
			IO.PUCR1.BIT.B7 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] |= 0x80;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		case _S_OUT_:
			// disable IRQ
			IO.PMR1.BIT.IRQ0 = _BA_PIN_MODE_IO_;
			bap_bpcrshadowregs [_PCR1_SHADOW_] |= 0x10;
			IO.PCR1 = bap_bpcrshadowregs [_PCR1_SHADOW_];
			break;
		#endif
	}
#endif	
}

// set the pin to the requested state
void setpin (u8 _bpin, u8 _bstate)
{
	if (_bstate == _LOW_)
		low (_bpin);
	else
		high (_bpin);
}

// set the pin low
void low (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM)))
	{
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_LOW(pi);
	}
}

// set the pin high
void high (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM)))
	{
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_HIGH(pi);
	}
}

// set the pin high
void toggle (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM)))
	{
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			PI_TOGGLE(pi);
	}
}


// get the state of the pin
u8 in (u8 _bpin)
{
	// Lets use our table to do the work for us...
	if (_bpin < (sizeof(g_PortTable)/sizeof(PORTITEM)))
	{
		// Ok lets get the port table item...
		PORTITEM pi = g_PortTable[_bpin];
		
		// Make sure it is a valid one...
		if (pi.pPDR)
			return PI_ISHIGH(pi);
	}
	return 0;	
}



//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//==============================================================================
// [Preliminary sound stuff] - move to approriate place...
//==============================================================================
// BUGBUG:: move to somewhere...
void Sound (u8 bPin, unsigned long wDur, unsigned long wFreq)			// dur in mS and Freq in Herz.  or should it be like basic???
{
	PORTITEM	piPin;									// The port item for the specified pin
	unsigned long	cCycles;								// the number of cycles to make this sound
	register unsigned long cPerHalfCycle;				// how many times to loop through per half cycle
	register unsigned long i;							// counter to use for the loops
	piPin = g_PortTable[bPin];							// get the port table item for the specified pin...
	PI_LOW(piPin);
	output(bPin);										// set the pin to output.
	
	// first calculate how many cycles of the tone we will produce
	cPerHalfCycle = (unsigned long)16000000L/(unsigned long)(wFreq * 2 *8)-3;			// *2(halves) *8(2dec, 2mov, 4bne) guess per count  - guess for setup and high/low
	cCycles = ((unsigned long)wDur * wFreq)/1000;

	while (cCycles--)
	{
		i = cPerHalfCycle;
		while (--i)	;
		PI_HIGH(piPin);
		i = cPerHalfCycle;
		while (--i)	;
		PI_LOW(piPin);										// leave low
	} 	

}

void MSound(u8 bPin, u8 cNotes, ...)
{
	va_list ap;
	word uDur;
	word uFreq;
	va_start(ap, cNotes);

	while (cNotes > 0)
	{
		uDur = va_arg(ap, unsigned int);
		uFreq = va_arg(ap, unsigned int);
		Sound(bPin, uDur, uFreq);
		cNotes--;
	}
	va_end(ap);
}

#ifdef __cplusplus
}
#endif
