//==============================================================================
// IOPINS - Handle some of the IO functions of the BAP - This is a merge of 
// 		some of definitions/functions from Minionbuilder (bap.h/c) and my
// 		work to integrate it with my own stuff and includes my speed up code
//==============================================================================
#ifndef _IOPINS_H_
#define _IOPINS_H_
#ifdef __cplusplus
extern "C" {
#endif

//==============================================================================
// Type Definitions and Macros
//==============================================================================

// Define some constants for our IO pins
enum {
	P0=0, P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11, P12, P13, P14, P15,
	P16, P17, P18, P19, P20, P21, P22, P23, P24, P25, P26, P27, P28, P29, P30, P31, _S_IN_, _S_OUT_,_S_HSERIAL_=(u8)-1};

// OK lets try to define the C version of the Port Table...
typedef struct {
		volatile u8					*pPDR;			// The PDR of the pin.
		u8							bMask;			// The mask in the IO register
		u8							bPU_PM;			// Pin have PUCR register(high nibble) PMDR(low nibble)
} PORTITEM;

#define PI_LOW(pi)		(*(pi.pPDR) &= ~(pi.bMask))
#define PI_HIGH(pi) 	(*(pi.pPDR) |= pi.bMask)
#define PI_TOGGLE(pi)	(*(pi.pPDR) ^= pi.bMask)
#define PI_ISHIGH(pi)	(*(pi.pPDR) & pi.bMask)

// Notes on bPU_PM high nibble is the offset from PUCR1(-1) (0-> no Pull up register)
#define PUR_1			0x10						// uses PUDR1
#define PUR_5			0x20						// uses PUDR5

// Likewise low nibble is offset from PMCR1(-1)  Assume that same mask will work!
#define PMR_1			0x1							// uses PMR1
#define PMR_5			0x2							// uses PMR5

//==============================================================================
// Function Prototypes
//==============================================================================

extern void input (u8 _bpin, bool fPullup=false);
extern void output (u8 _bpin);
extern void low (u8 _bpin);
extern void high (u8 _bpin);
extern void toggle (u8 _bpin);
extern void setpin (u8 _bpin, u8 _bstate);
extern u8 in(u8 _bpin);


// These should maybe go to their own file?
extern void pauseus (unsigned long _lpause);
extern void pause (unsigned long  _lpause);
extern void Sound (u8 bPin, unsigned long wDur, unsigned long wFreq);			// dur in mS and Freq in Herz.  or should it be like basic???
extern void MSound(u8 bPin, u8 cNotes, ...);

//==============================================================================
// Global variable definitions.
//==============================================================================
extern const PORTITEM	g_PortTable[];				// used to map logical pins to IOPort and mask


#ifdef __cplusplus
}
#endif
#endif /* _IOPINS_H_ */

